@@ -10,10 +10,11 @@
 
            #:passwd-name #:passwd-passwd #:passwd-uid #:passwd-gid
            #:passwd-gecos #:passwd-dir #:passwd-shell
+           #:group-name #:group-gid #:group-passwd
            #:stat-mode #:stat-ino #:stat-dev #:stat-nlink #:stat-uid
            #:stat-gid #:stat-size #:stat-atime #:stat-mtime #:stat-ctime
            #:termios-iflag #:termios-oflag #:termios-cflag
-           #:termios-lflag #:termios-cc))
+           #:termios-lflag #:termios-cc #:timeval-sec #:timeval-usec))
 
 #+win32
 (load-shared-object "msvcrt.dll")
