@@ -96,6 +96,7 @@
         *gc-inhibit* t
         *gc-pending* nil
         #!+sb-thread *stop-for-gc-pending* #!+sb-thread nil
+        *allow-with-interrupts* t
         *interrupts-enabled* t
         *interrupt-pending* nil
         *break-on-signals* nil
@@ -103,7 +104,8 @@
         *current-error-depth* 0
         *cold-init-complete-p* nil
         *type-system-initialized* nil
-        sb!vm:*alloc-signal* nil)
+        sb!vm:*alloc-signal* nil
+        sb!kernel::*gc-epoch* (cons nil nil))
 
   ;; I'm not sure where eval is first called, so I put this first.
   #!+sb-eval
@@ -289,15 +291,15 @@ UNIX-like systems, UNIX-STATUS is used as the status code."
 (defun reinit ()
   (setf *default-external-format* nil)
   (setf sb!alien::*default-c-string-external-format* nil)
-  (without-interrupts
-    (without-gcing
-        (os-cold-init-or-reinit)
-      (thread-init-or-reinit)
-      (stream-reinit)
-      #!-win32
-      (signal-cold-init-or-reinit)
-      (setf (sb!alien:extern-alien "internal_errors_enabled" boolean) t)
-      (float-cold-init-or-reinit)))
+  ;; WITHOUT-GCING implies WITHOUT-INTERRUPTS.
+  (without-gcing
+    (os-cold-init-or-reinit)
+    (thread-init-or-reinit)
+    (stream-reinit t)
+    #!-win32
+    (signal-cold-init-or-reinit)
+    (setf (sb!alien:extern-alien "internal_errors_enabled" boolean) t)
+    (float-cold-init-or-reinit))
   (gc-reinit)
   (foreign-reinit)
   (time-reinit)
