@@ -208,9 +208,9 @@
                                            &body body)
   #!+sb-doc
   "Within the lexical scope of the body forms, MNAME is defined via macrolet
-   such that successive invocations of (MNAME) will return the symbols,
-   one by one, from the packages in PACKAGE-LIST. SYMBOL-TYPES may be
-   any of :INHERITED :EXTERNAL :INTERNAL."
+such that successive invocations of (MNAME) will return the symbols, one by
+one, from the packages in PACKAGE-LIST. SYMBOL-TYPES may be any
+of :INHERITED :EXTERNAL :INTERNAL."
   (let* ((packages (gensym))
          (these-packages (gensym))
          (ordered-types (let ((res nil))
